//--------------------------------------------------------------------------------------------------
//
// Title       : Memory Mapped Serial Interface
// Design      : single
// Author      : Shen-Hui Lee
// Company     : UW
//
//-------------------------------------------------------------------------------------------------
//
// File        : Z:\pipelined\compile\MemoryMappedSerial.v
// Generated   : Tue May  2 11:33:43 2006
// From        : Z:\pipelined\src\Bootloader\MemoryMappedSerial.bde
// By          : Bde2Verilog ver. 2.01
//
//-------------------------------------------------------------------------------------------------
//
// Description : 
//
//-------------------------------------------------------------------------------------------------

`ifdef _VCP
`else
`define library
`endif


// ---------- Design Unit Header ---------- //
`timescale 1ps / 1ps

module MemoryMappedSerial_TF (ProcessorClock,clk,reset,SerialIOAddress,SerialIOData,SerialIOWE,DataOut) ;

// ------------ Port declarations --------- //
input ProcessorClock;
wire ProcessorClock;
input clk;
wire clk;
input reset;
wire reset;
input [5:0] SerialIOAddress;
wire [5:0] SerialIOAddress;
input [31:0] SerialIOData;
wire [31:0] SerialIOData;
input [1:0] SerialIOWE;
wire [1:0] SerialIOWE;
output [31:0] DataOut;
wire [31:0] DataOut;

// ----------- Signal declarations -------- //
wire NET1441;
wire NET2020;
wire ProcClock;
wire RxEmpty;
wire RxFIFOFull;
wire RxFIFORd;
wire RxFIFOWE;
wire sendFIFORead;
wire TxD_Busy;
wire TxD_Start;
wire TxFIFOFull;
wire TxFIFOWE;
wire [39:0] BUS1429;
wire [7:0] BUS1970;
wire [39:0] RxDataIn;
wire [39:0] RxFIFOOut;
wire [7:0] TxData;
wire [39:0] TxDIN;		 

parameter PROGRAM = "serial_tf.hex";
parameter PROGRAM_SIZE = 1024;

// -------- Component instantiations -------//
`library("InterfaceManager","lib378")
MemoryMappedSerialManager InterfaceManager
(
	.InAddress(SerialIOAddress),
	.InData(SerialIOData),
	.InWE(SerialIOWE),
	.OutData(DataOut),
	.RecvFIFOEmpty(RxEmpty),
	.RecvFIFORead(RxFIFORd),
	.RecvFIFOTop(RxFIFOOut),
	.SendFIFOFull(TxFIFOFull),
	.SendFIFOWrite(TxFIFOWE),
	.clk(ProcClock),
	.reset(reset),
	.sendFIFOData(TxDIN)
); 

serial_tf
#(
.PRGM(PROGRAM),
.PRGM_SIZE(PROGRAM_SIZE))
TestFixture
( 
.CLK(ProcClock),
.reset(reset),
.read_do(RxFIFORd),
.read_head(RxFIFOOut),
.read_empty(RxEmpty),
.write_do(TxFIFOWE),
.write_data(TxDIN),
.write_full(TxFIFOFull)
);

// ----------- Terminals assignment --------//
//	       ---- Input terminals ---         //
assign ProcClock = ProcessorClock;

endmodule 

module serial_tf(CLK, reset, read_do, read_head, read_empty, write_do, write_data, write_full);
	parameter PRGM = "serial_tf.hex";
	parameter PRGM_SIZE = 1024;
	
	input CLK, reset;
	
	input read_do;
	output read_empty;
	output [39:0] read_head;
	
	input write_do;
	output write_full;
	input [39:0] write_data;
	
	reg [39:0] prgm [0:PRGM_SIZE];
	integer cmds_read, i;
	
	initial begin
		cmds_read = 0;
		
		// clear out program memory
		for (i = 0; i < PRGM_SIZE; i = i + 1) begin
			prgm[i] = 40'b0;
		end
		
		// load program from PRGM
		$readmemh(PRGM, prgm);
	end
	
	always @(posedge CLK) begin
		if (reset) cmds_read = 0;
		if (read_do) cmds_read = cmds_read + 1;
		if (write_do) $display("serial_tf: word 0x%H written to the serial output", write_data);
	end
	
	assign read_head = (cmds_read >= PRGM_SIZE) ? 40'hx : prgm[cmds_read];
	assign read_empty = (cmds_read >= PRGM_SIZE);
	
	assign write_full = 0;
endmodule